package edu.washington.cs.radishsim;

import edu.washington.cs.radishsim.Machine.MachineParams;
import edu.washington.cs.radishsim.Machine.SimulationMode;

/**
 * Setup some default parameters used by the test cases. Each suite overrides these values as
 * necessary.
 */
public abstract class DefaultTestingMachineParams extends MachineParams<RadishLine> {

   @Override
   SimulationMode simulationMode() {
      return SimulationMode.RADISH;
   }

   @Override
   boolean useL2() {
      return false;
   }

   @Override
   CacheConfiguration<RadishLine> l2config() {
      return null;
   }

   @Override
   boolean useL3() {
      return false;
   }

   @Override
   CacheConfiguration<RadishLine> l3config() {
      return null;
   }

   @Override
   LineFactory<RadishLine> lineFactory() {
      return new LineFactory<RadishLine>() {
         @Override
         public RadishLine create() {
            return new RadishLine(RadishLine.Compacted.EXPANDED);
         }

         @Override
         public RadishLine create(LineAddress la, RadishLine.Compacted c) {
            return new RadishLine(RadishLine.Compacted.EXPANDED, la);
         }
      };
   }

   @Override
   boolean useWholeLineSoftwareChecks() {
      return true;
   }

   @Override
   boolean useClockGC() {
      return false;
   }

   @Override
   boolean useTypeSafetyOpt() {
      return false;
   }

   @Override
   boolean remoteAccessesAffectLRU() {
      return false;
   }

   @Override
   int swCheckCost() {
      return 0;
   }

   @Override
   int hwCheckCost() {
      return 0;
   }

   @Override
   int epochRolloverCost() {
      return 0;
   }
   
   @Override
   boolean evictUselessMetadata() {
      return false;
   }
   
   @Override
   Machine.LeafReadConfiguration leafReads() {
      return Machine.LeafReadConfiguration.LAZY_LEAF_READS;
   }
   
   @Override
   boolean metadataLLCEvictionPolicy() {
      return false;
   }
   
   @Override
   int asyncProcessingCores() {
      return 0;
   }
   
   @Override
   boolean simulateSoftwareChecks() {
      return false;
   }
   
   @Override
   boolean compactMetadata() {
      return false;
   }
   
   @Override
   int cacheUsageSnapshotFrequencyInCycles() {
      return 100;
   }
}
